#! /usr/bin/env python
# -*- coding: utf-8 -*-

import time
from .. import es

class BftWorks(object):

	RiseThreshold = 0.3

	ShowMax = {1: 50, 2: 100, 3: 500}

	Weights = {1: {"like": 1.5, "share": 1.5, "longPlay": 1, "play": 1}
	, 2: {"like": 2, "share": 2, "longPlay": 1, "play": 1}
	, 3: {"like": 2, "share": 3, "longPlay": 1, "play": 1}
	, 4: {"like": 2, "share": 3, "longPlay": 1, "play": 1}
	, 5: {"like": 2, "share": 3, "longPlay": 1, "play": 1}}

	#1 一层流量池 2 二层流量池 3 三层流量池 4 晋级池 5 淘汰池
	Pools = {'FlowFirst': 1, 'FlowSecond': 2, 'FlowThird': 3, 'Success': 4, 'Fail': 5}

	def __init__(self, vid):
		self.vid = vid
		self.score = 0
		self.enterPool(BftWorks.Pools['FlowFirst'])

	def updateAction(self, action):
		if self.pool not in [BftWorks.Pools['FlowFirst'], BftWorks.Pools['FlowSecond'], BftWorks.Pools['FlowThird']]:
			return
		actionId = action['action']
		uid = action['uid']
		if actionId == 3:
			self.uidSetDict['show'].add(uid)
		elif actionId == 16:
			self.uidSetDict['like'].add(uid)
		elif actionId == 12 or actionId == 13:
			self.uidSetDict['share'].add(uid)
		elif actionId == 2:
			self.uidSetDict['longPlay'].add(uid)
		elif actionId == 5:
			self.uidSetDict['play'].add(uid)

	def updateStatus(self):
		if self.pool not in [BftWorks.Pools['FlowFirst'], BftWorks.Pools['FlowSecond'], BftWorks.Pools['FlowThird']]:
			return
		showMax = BftWorks.ShowMax[self.pool]
		weight = BftWorks.Weights[self.pool]

		show = self.getCount('show')
		score = 0
		totalOne = 0
		for key in weight.keys():
			score += weight[key] * self.getCount(key)
			totalOne += weight[key]
		
		if score >= max(showMax, show) * totalOne * BftWorks.RiseThreshold:
			self.enterPool(self.pool + 1)
		elif show >= showMax:
			self.oldPool = self.pool
			self.enterPool(BftWorks.Pools['Fail'])
		self.score = score

	def init(self, endTime):
		body = {"query": {"bool": {"must": [{"range": {"time": {"gte": self.timeEnter, "lt": endTime}}}, {"term": {"vid": self.vid}}, {"terms": {"action": [2, 3, 5, 12, 13, 16]}}]}}, "size": 0, "aggs": {}}
		body['aggs']['uid_diff'] = {"terms": {"field": "action", "size": 6}, "aggs": {"count": {"cardinality": {"field": "uid"}}}}
		aggs = es.searchStatisticsAggs('works_actions', body)
		
		counts = {2: 0, 3: 0, 5: 0, 12: 0, 13: 0, 16: 0}
		for bucket in aggs['uid_diff']['buckets']:
			counts[bucket['key']] = bucket['count']['value']

		self.initData['show'] = counts[3]
		self.initData['like'] = counts[16]
		self.initData['share'] = counts[12] + counts[13]
		self.initData['longPlay'] = counts[2]
		self.initData['play'] = counts[5]

		self.updateStatus()

	def enterPool(self, newPool):
		self.pool = newPool
		self.timeEnter = es.timeToEsLocalTime(time.time())
		self.initData = {'show': 0, 'like': 0, 'share': 0, 'longPlay': 0, 'play': 0}
		self.uidSetDict = {'show': set(), 'like': set(), 'share': set(), 'longPlay': set(), 'play': set()}

	def getCount(self, key):
		return self.initData[key] + len(self.uidSetDict[key])

	def toSaveEsData(self):
		data = {"vid": self.vid, "pool": self.pool, "timeEnter": self.timeEnter, 'score': self.score, 'show': self.getCount('show')}
		for key in BftWorks.Weights[self.pool].keys():
			data[key] = self.getCount(key)
		return data

	def syncEsData(self, source):
		self.pool = source["pool"]
		self.timeEnter = source["timeEnter"]
	


	


#! /usr/bin/env python
# -*- coding: utf-8 -*-

import time, random, math
from .. import es
from bftWorks import BftWorks

PoolCountKey = {1: 'flowFirstCount', 2: 'flowSecondCount', 3: 'flowThirdCount', 4: 'successCount', 5: 'failCount'}

def updateWorks():
	#endTime = int(time.time())
	lastDay = 1547568000

	endTime = 1547568000
	vids = []
	for index in range(0, 80):
		vids.append(str(index))
	#vids = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23', '24', '25', '26', '27', '28', '29']
	worksDict = getWorksDict(vids, endTime)

	while True:
		if random.randint(1, 2) == 1:
			vids.append(str(time.time()))
		worksDict = updateWorksDict(worksDict, vids, endTime)
		endTime = update(worksDict, endTime)
		
		if endTime - lastDay >= 24 * 60 * 60:
			Pools = BftWorks.Pools
			failWorks = {Pools['FlowFirst']: [], Pools['FlowSecond']: [], Pools['FlowThird']: []}
			for works in worksDict.values():
				if works.pool != Pools['Fail']:
					continue
				failWorks[works.oldPool].append(works)
			for pool, worksList in failWorks.items():
				worksList.sort(reverse=True, key=lambda item: item.score)
				for index in range(0, int(math.ceil(len(worksList) * 0.2))):
					worksList[index].enterPool(pool + 1)
			lastDay = endTime

		saveWorksToEs(worksDict)

		Pools = BftWorks.Pools
		record = {'time': es.timeToEsLocalTime(endTime), 'flowFirstCount': 0, 'flowSecondCount': 0, 'flowThirdCount': 0, 'successCount': 0, 'failCount': 0}
		show = {Pools['FlowFirst']: 0, Pools['FlowSecond']: 0, Pools['FlowThird']: 0}
		for works in worksDict.values():
			record[PoolCountKey[works.pool]] += 1
			if works.pool in [Pools['FlowFirst'], Pools['FlowSecond'], Pools['FlowThird']]:
				show[works.pool] += works.getCount('show')
		record['flowFirstShowAvg'] = 0 if record['flowFirstCount'] == 0 else show[Pools['FlowFirst']] / record['flowFirstCount']
		record['flowSecondShowAvg'] = 0 if record['flowSecondCount'] == 0 else show[Pools['FlowSecond']] / record['flowSecondCount']
		record['flowThirdShowAvg'] = 0 if record['flowThirdCount'] == 0 else show[Pools['FlowThird']] / record['flowThirdCount']
		es.updateStatisticsData('works_filter_pool_record', {int(endTime): record})

		print(es.timeToEsLocalTime(endTime))
		#time.sleep(300)

def update(worksDict, startTime):
	#endTime = int(time.time() - 90)
	endTime = startTime + 10 * 60
	for hit in es.searchStatisticsDataByBody("works_actions", {"query": {"bool": {"must": [{"range": {"time": {"gte": startTime * 1000, "lt": endTime * 1000}}}, {"terms": {"action": [2, 3, 5, 12, 13, 16]}}]}}, "size": 10000}):
		action = hit['_source']
		vid = action['vid']
		if random.randint(1, 5) == 1:
			for i in range(0, 100):
				vid = random.sample(worksDict.keys(), 1)[0]
				if worksDict[vid].pool in [BftWorks.Pools['FlowFirst'], BftWorks.Pools['FlowSecond'], BftWorks.Pools['FlowThird']]:
					break
		if worksDict.has_key(vid):
			worksDict[vid].updateAction(action)

	for works in worksDict.values():
		works.updateStatus()

	return endTime

def updateWorksDict(worksDict, vids, endTime):
	newWorksDict = {}
	for vid in vids:
		if worksDict.has_key(vid):
			newWorksDict[vid] = worksDict[vid]
		else:
			works = BftWorks(vid)
			works.init(es.timeToEsLocalTime(endTime))
			newWorksDict[vid] = works
	return newWorksDict

def getWorksDict(vids, endTime):
	worksDict = {}
	for vid in vids:
		worksDict[vid] = BftWorks(vid)
	syncEsWorks(worksDict)
	for works in worksDict.values():
		works.init(es.timeToEsLocalTime(endTime))
	return worksDict

def syncEsWorks(worksDict):
	for hit in es.searchStatisticsData('works_filter_pool'):
		vid = hit['_id']
		if worksDict.has_key(vid):
			worksDict[vid].syncEsData(hit['_source'])

def saveWorksToEs(worksDict):
	worksData = {}
	for vid, works in worksDict.items():
		worksData[vid] = works.toSaveEsData()
	es.updateStatisticsData('works_filter_pool', worksData)

	


#! /usr/bin/env python
# -*- coding: utf-8 -*-

import time
from .. import es, dataDispose

def test():
	statisticsRecConv()

def statisticsRecConv():
	convTotal = 0
	uids = []
	for hit in es.searchStatisticsDataByBody('works_actions', {"query": {"bool": {"must": [{"range": {"time": {"gte": 1547049600000, "lt": 1547136000000}}}, {"term": {"action": {"value": 3}}}]}}, "size":10000}):
		uid = hit["_source"]["uid"]
		if uid in uids:
			continue
		uids.append(uid)
		body = {"query": {"bool": {"must": [{"term": {"uid": uid}}, {"terms": {"action": [3, 5]}}, {"range": {"time": {"gte": 1547049600000, "lt": 1547136000000}}}]}}, "size": 0, "aggs": {}}
		body['aggs']['vid_diff'] = {"terms": {"field": "action", "size": 2}, "aggs": {"count": {"cardinality": {"field": "vid"}}}}
		aggs = es.searchStatisticsAggs('works_actions', body)
		recShow = 0
		recShowPlay = 0
		for bucket in aggs['vid_diff']['buckets']:
			if bucket['key'] == 3:
				recShow = bucket['count']['value']
			elif bucket['key'] == 5:
				recShowPlay = bucket['count']['value']
		convTotal += (0 if recShow == 0 else recShowPlay * 1.0 / recShow)
		count = len(uids)
		print(count, uid, recShow, recShowPlay, convTotal, convTotal * 1.0 / count)
		time.sleep(1)



def testA():
	worksList = dataDispose.getWorksList()
	data = {}
	for index, works in enumerate(worksList):
		print(index, works.vid)
		body = {"query": {"bool": {"must": [{"term": {"vid": works.vid}}, {"terms": {"action": [3, 5]}}]}}, "size": 0, "aggs": {}}
		body['aggs']['uid_diff'] = {"terms": {"field": "action", "size": 2}, "aggs": {"count": {"cardinality": {"field": "uid"}}}}
		aggs = es.searchStatisticsAggs('works_actions', body)
		data[works.vid] = {'vid': works.vid, 'aid': works.aid, 'title': works.title, 'resShow': 0, 'resShowPlay': 0}
		for bucket in aggs['uid_diff']['buckets']:
			if bucket['key'] == 3:
				data[works.vid]['resShow'] = bucket['count']['value']
			elif bucket['key'] == 5:
				data[works.vid]['resShowPlay'] = bucket['count']['value']

	#print(data)
	es.updateStatisticsData('works_info', data)


	'''
	values = []
	for key, value in {'a': 1, 'b': 2}.items():
		values += [key, value]
	testArgs(**{'a': 1, 'b': 2})
	'''
	'''
	es.setEnv(False)
	hits = es.searchStatisticsDataByBody('works_actions', {"query": {"term": {"vid": "ecbf3aa1599e140896e0f6c241d93681"}}, "size": 10000})
	
	dataDict = {}
	for hit in hits:
		dataDict[hit['_id']] = hit['_source']

	es.setEnv(True)
	es.updateStatisticsData('works_actions', dataDict)
	'''

def testArgs(*args, **kwargs):
	print(args)
	print(kwargs)